@@ -153,7 +153,7 @@ nsPrefListener.prototype =
           editor.isCSSEnabled = useCSS;
       }
     }
-    else if (prefName.substr(0, kEditorToolbarPrefs.length) == kEditorToolbarPrefs)
+    else if (prefName.startsWith(kEditorToolbarPrefs))
     {
       let id = prefName.substr(kEditorToolbarPrefs.length) + "Button";
       let button = document.getElementById(id);
@@ -261,7 +261,7 @@ var gEditorDocumentObserver =
             // network code popped up an alert dialog, so we don't need to
           }
           if (errorStringId)
-            AlertWithTitle("", GetString(errorStringId));
+            Services.prompt.alert(window, "", GetString(errorStringId));
         } catch(e) { dump("EXCEPTION GETTING obs_documentCreated state "+e+"\n"); }
 
         // We have a bad editor -- nsIEditingSession will rebuild an editor
@@ -345,17 +345,9 @@ var gEditorDocumentObserver =
             HideItem("structSpacer");
 
             // Hide everything in "Insert" except for "Symbols"
-            var menuPopup = document.getElementById("insertMenuPopup");
-            if (menuPopup)
-            {
-              var children = menuPopup.childNodes;
-              for (var i=0; i < children.length; i++) 
-              {
-                var item = children.item(i);
-                if (item.id != "insertChars")
-                  item.hidden = true;
-              }
-            }
+            let menuPopupChildren = document.querySelectorAll('[id="insertMenuPopup"] > :not(#insertChars)');
+            for (let i = 0; i < menuPopupChildren.length; i++)
+              menuPopupChildren.item(i).hidden = true;
           }
     
           // Set window title
@@ -490,80 +482,6 @@ function EditorSharedStartup()
   gColorObj.LastHighlightColor = "";
 }
 
-function toggleAffectedChrome(aHide)
-{
-  // chrome to toggle includes:
-  //   (*) menubar
-  //   (*) toolbox
-  //   (*) sidebar
-  //   (*) statusbar
-
-  if (!gChromeState)
-    gChromeState = new Object;
-
-  var statusbar = document.getElementById("status-bar");
-
-  // sidebar states map as follows:
-  //   hidden    => hide/show nothing
-  //   collapsed => hide/show only the splitter
-  //   shown     => hide/show the splitter and the box
-  if (aHide)
-  {
-    // going into print preview mode
-    gChromeState.sidebar = SidebarGetState();
-    SidebarSetState("hidden");
-
-    // deal with the Status Bar
-    gChromeState.statusbarWasHidden = statusbar.hidden;
-    statusbar.hidden = true;
-  }
-  else
-  {
-    // restoring normal mode (i.e., leaving print preview mode)
-    SidebarSetState(gChromeState.sidebar);
-
-    // restore the Status Bar
-    statusbar.hidden = gChromeState.statusbarWasHidden;
-  }
-
-  // if we are unhiding and sidebar used to be there rebuild it
-  if (!aHide && gChromeState.sidebar == "visible")
-    SidebarRebuild();
-
-  document.getElementById("EditorToolbox").hidden = aHide;
-  document.getElementById("appcontent").collapsed = aHide;
-}
-
-var PrintPreviewListener = {
-  getPrintPreviewBrowser: function () {
-    var browser = document.getElementById("ppBrowser");
-    if (!browser) {
-      browser = document.createElement("browser");
-      browser.setAttribute("id", "ppBrowser");
-      browser.setAttribute("flex", "1");
-      browser.setAttribute("disablehistory", "true");
-      browser.setAttribute("disablesecurity", "true");
-      browser.setAttribute("type", "content");
-      document.getElementById("sidebar-parent").
-        insertBefore(browser, document.getElementById("appcontent"));
-    }
-    return browser;
-  },
-  getSourceBrowser: function () {
-    return GetCurrentEditorElement();
-  },
-  getNavToolbox: function () {
-    return document.getElementById("EditorToolbox");
-  },
-  onEnter: function () {
-    toggleAffectedChrome(true);
-  },
-  onExit: function () {
-    document.getElementById("ppBrowser").collapsed = true;
-    toggleAffectedChrome(false);
-  }
-}
-
 // This method is only called by Message composer when recycling a compose window
 function EditorResetFontAndColorAttributes()
 {
@@ -652,42 +570,35 @@ function CheckAndSaveDocument(command, allowDontSave)
   var dialogMsg = GetString(doPublish ? "PublishPrompt" : "SaveFilePrompt");
   dialogMsg = (dialogMsg.replace(/%title%/,title)).replace(/%reason%/,reasonToSave);
 
-  var promptService = GetPromptService();
-
-  var result = {value:0};
-  var promptFlags = promptService.BUTTON_TITLE_CANCEL * promptService.BUTTON_POS_1;
-  var button1Title = null;
-  var button3Title = null;
+  let result = {value:0};
+  let promptFlags = Services.prompt.BUTTON_TITLE_CANCEL * Services.prompt.BUTTON_POS_1;
+  let button1Title = null;
+  let button3Title = null;
 
   if (doPublish)
   {
-    promptFlags += promptService.BUTTON_TITLE_IS_STRING * promptService.BUTTON_POS_0;
+    promptFlags += Services.prompt.BUTTON_TITLE_IS_STRING * Services.prompt.BUTTON_POS_0;
     button1Title = GetString("Publish");
     button3Title = GetString("DontPublish");    
   }
   else
   {
-    promptFlags += promptService.BUTTON_TITLE_SAVE * promptService.BUTTON_POS_0;
+    promptFlags += Services.prompt.BUTTON_TITLE_SAVE * Services.prompt.BUTTON_POS_0;
   }
 
   // If allowing "Don't..." button, add that
   if (allowDontSave)
     promptFlags += doPublish ?
-        (promptService.BUTTON_TITLE_IS_STRING * promptService.BUTTON_POS_2)
-        : (promptService.BUTTON_TITLE_DONT_SAVE * promptService.BUTTON_POS_2);
+        (Services.prompt.BUTTON_TITLE_IS_STRING * Services.prompt.BUTTON_POS_2)
+        : (Services.prompt.BUTTON_TITLE_DONT_SAVE * Services.prompt.BUTTON_POS_2);
   
-  result = promptService.confirmEx(window, dialogTitle, dialogMsg, promptFlags,
+  result = Services.prompt.confirmEx(window, dialogTitle, dialogMsg, promptFlags,
                           button1Title, null, button3Title, null, {value:0});
 
   if (result == 0)
   {
     // Save, but first finish HTML source mode
-    if (IsHTMLSourceChanged()) {
-      try {
-        FinishHTMLSource();
-      } catch (e) { return false;}
-    }
-
+    SetEditMode(gPreviousNonSourceDisplayMode);
     if (doPublish)
     {
       // We save the command the user wanted to do in a global
@@ -1561,7 +1472,7 @@ function SetEditMode(mode)
           if (dt.publicId)
             doctypeText += " PUBLIC \"" + domdoc.doctype.publicId;
           if (dt.systemId)
-            doctypeText += " "+"\"" + dt.systemId;
+            doctypeText += " \"" + dt.systemId;
           doctypeText += "\">"
           doctypeNode.setAttribute("value", doctypeText);
         }
@@ -1632,15 +1543,8 @@ function SetEditMode(mode)
 
         // Get the text for the <title> from the newly-parsed document
         // (must do this for proper conversion of "escaped" characters)
-        var title = "";
-        var titlenodelist = editor.document.getElementsByTagName("title");
-        if (titlenodelist)
-        {
-          var titleNode = titlenodelist.item(0);
-          if (titleNode && titleNode.firstChild && titleNode.firstChild.data)
-            title = titleNode.firstChild.data;
-        }
-        SetDocumentTitle(title);
+        let titleNode = editor.document.querySelector("title");
+        SetDocumentTitle(titleNode ? titleNode.textContent : "");
 
       } catch (ex) {
         dump(ex);
@@ -1688,30 +1592,6 @@ function CancelHTMLSource()
   SetDisplayMode(gPreviousNonSourceDisplayMode);
 }
 
-function FinishHTMLSource()
-{
-  var editor = GetCurrentEditor();
-
-  if (IsInHTMLSourceMode() && editor.contentsMIMEType == kXHTMLMimeType)
-  {
-    var html = null;
-    try {
-      var htmlSource = gSourceTextEditor.outputToString(kTextMimeType,
-                                                        kOutputLFLineBreak);
-      editor.document.createRange().createContextualFragment(htmlSource);
-    } catch (e) {
-      AlertWithTitle(GetString("Alert"), GetString("Malformed"));
-      //cheat to force back to Source Mode
-      gEditorDisplayMode = kDisplayModePreview;
-      SetDisplayMode(kDisplayModeSource);
-      throw e;
-    }
-  }
-
-  // Switch edit modes -- converts source back into DOM document
-  SetEditMode(gPreviousNonSourceDisplayMode);
-}
-
 function SetDisplayMode(mode)
 {
   if (!IsHTMLEditor())
@@ -2139,15 +2019,8 @@ function EditorSetDefaultPrefsAndDoctype()
   }
   
   // search for head; we'll need this for meta tag additions
-  var headelement = 0;
-  var headnodelist = domdoc.getElementsByTagName("head");
-  if (headnodelist)
-  {
-    var sz = headnodelist.length;
-    if ( sz >= 1 )
-      headelement = headnodelist.item(0);
-  }
-  else
+  let headelement = domdoc.querySelector("head");
+  if (!headelement)
   {
     headelement = domdoc.createElement("head");
     if (headelement)
@@ -2163,8 +2036,7 @@ function EditorSetDefaultPrefsAndDoctype()
   // if not, create one and make it a child of the head tag
   //   and set its content attribute to the value of the editor.author preference.
 
-  var nodelist = domdoc.getElementsByTagName("meta");
-  if ( nodelist )
+  if (domdoc.querySelector("meta"))
   {
     // we should do charset first since we need to have charset before
     // hitting other 8-bit char in other meta tags
@@ -2180,50 +2052,31 @@ function EditorSetDefaultPrefsAndDoctype()
     if ( prefCharsetString && prefCharsetString != 0)
       editor.documentCharacterSet = prefCharsetString;
 
-    var node = 0;
-    var listlength = nodelist.length;
-
     // let's start by assuming we have an author in case we don't have the pref
-    var authorFound = false;
-    for (var i = 0; i < listlength && !authorFound; i++)
-    {
-      node = nodelist.item(i);
-      if ( node )
-      {
-        var value = node.getAttribute("name");
-        if (value && value.toLowerCase() == "author")
-        {
-          authorFound = true;
-        }
-      }
-    }
 
-    var prefAuthorString = 0;
+    var prefAuthorString = null;
+    let authorFound = domdoc.querySelector('meta[name="author"]');
     try
     {
       prefAuthorString = Services.prefs.getComplexValue("editor.author",
                                                         Components.interfaces.nsISupportsString).data;
     }
     catch (ex) {}
-    if ( prefAuthorString && prefAuthorString != 0)
+    if (prefAuthorString && prefAuthorString != 0 && !authorFound && headelement)
     {
-      if ( !authorFound && headelement)
+      // create meta tag with 2 attributes
+      element = domdoc.createElement("meta");
+      if (element)
       {
-        /* create meta tag with 2 attributes */
-        element = domdoc.createElement("meta");
-        if ( element )
-        {
-          element.setAttribute("name", "author");
-          element.setAttribute("content", prefAuthorString);
-          headelement.appendChild( element );
-        }
+        element.setAttribute("name", "author");
+        element.setAttribute("content", prefAuthorString);
+        headelement.appendChild(element);
       }
     }
   }
 
   // add title tag if not present
-  var titlenodelist = editor.document.getElementsByTagName("title");
-  if (headelement && titlenodelist && titlenodelist.length == 0)
+  if (headelement && !editor.document.querySelector("title"))
   {
      var titleElement = domdoc.createElement("title");
      if (titleElement)
@@ -3102,26 +2955,10 @@ function RemoveTOC()
     elt.parentNode.removeChild(elt);
   }
 
-  function acceptNode(node)
-  {
-    if (node.nodeName.toLowerCase() == "a" &&
-        node.hasAttribute("name") &&
-        node.getAttribute("name").substr(0, 8) == "mozTocId") {
-      return NodeFilter.FILTER_ACCEPT;
-    }
-    return NodeFilter.FILTER_SKIP;
-  }
-
-  var treeWalker = theDocument.createTreeWalker(theDocument.documentElement,
-                                                NodeFilter.SHOW_ELEMENT,
-                                                acceptNode,
-                                                true);
-  if (treeWalker) {
-    var anchorNode = treeWalker.nextNode();
-    while (anchorNode) {
-      var tmp = treeWalker.nextNode();
-      anchorNode.parentNode.removeChild(anchorNode);
-      anchorNode = tmp;
+  let anchorNodes = theDocument.querySelectorAll('a[name^="mozTocId"]');
+  for (let node of anchorNodes) {
+    if (node.parentNode) {
+      node.parentNode.removeChild(node);
     }
   }
 }
