@@ -4,11 +4,12 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 Components.utils.import("resource:///modules/mailServices.js");
+Components.utils.import("resource:///modules/MailUtils.js");
 
 /**
  * Get the identity that most likely is the best one to use, given the hint.
- * @param identities nsISupportsArray<nsIMsgIdentity> of identities
- * @param optionalHint string containing comma separated mailboxes
+ * @param identities    nsIArray<nsIMsgIdentity> of identities
+ * @param optionalHint  string containing comma separated mailboxes
  */
 function getBestIdentity(identities, optionalHint)
 {
@@ -286,7 +287,7 @@ function Subscribe(preselectedMsgFolder)
 function SubscribeOKCallback(changeTable)
 {
   for (var serverURI in changeTable) {
-    var folder = GetMsgFolderFromUri(serverURI, true);
+    var folder = MailUtils.getFolderForURI(serverURI, true);
     var server = folder.server;
     var subscribableServer =
           server.QueryInterface(Components.interfaces.nsISubscribableServer);
@@ -400,9 +401,9 @@ function SaveAsTemplate(uri) {
     let templates = MailUtils.getFolderForURI(identity.stationeryFolder, false);
     if (!templates.parent) {
       templates.setFlag(Ci.nsMsgFolderFlags.Templates);
-      let isImap = templates.server.type == "imap";
+      let isAsync = templates.server.protocolInfo.foldersCreatedAsync;
       templates.createStorageIfMissing(new saveAsUrlListener(uri, identity));
-      if (isImap)
+      if (isAsync)
         return;
     }
     messenger.saveAs(uri, false, identity, null);
