@@ -121,11 +121,9 @@ class ThunderTestProfile(mozrunner.ThunderbirdProfile):
         # Do not allow check new mail to be set
         'mail.startup.enabledMailCheckOnce' :  True,
         # Disable compatibility checking
-        'extensions.checkCompatibility.22.0': False,
+        'extensions.checkCompatibility.27.0': False,
         # Stop any pings to AMO on add-on install
         'extensions.getAddons.cache.enabled': False,
-        # Disable test pilot new tab (this can be set to anything currently, just needs to be set).
-        'extensions.testpilot.lastversion': '1.0',
         # In case a developer is working on a laptop without a network
         # connection, don't detect offline mode; hence we'll still startup
         # online which is what mozmill currently requires. It'll also protect us
@@ -143,7 +141,7 @@ class ThunderTestProfile(mozrunner.ThunderbirdProfile):
         # Do not allow fonts to be upgraded
         'mail.font.windows.version': 2,
         # No, we don't want to be prompted about Telemetry
-        'toolkit.telemetry.prompted': True,
+        'toolkit.telemetry.prompted': 999,
         }
 
     # Dummied up local accounts to stop the account wizard
@@ -175,7 +173,7 @@ class ThunderTestProfile(mozrunner.ThunderbirdProfile):
         'mail.server.server2.check_new_mail' :  False,
         'mail.server.server2.directory-rel' :  "[ProfD]Mail/tinderbox",
         'mail.server.server2.download_on_biff' :  True,
-        'mail.server.server2.hostname' :  "tinderbox",
+        'mail.server.server2.hostname' :  "tinderbox123",
         'mail.server.server2.login_at_startup' :  False,
         'mail.server.server2.name' :  "tinderbox@foo.invalid",
         'mail.server.server2.type' :  "pop3",
@@ -186,7 +184,7 @@ class ThunderTestProfile(mozrunner.ThunderbirdProfile):
         'mail.server.server3.type' :  "im",
         'mail.server.server3.userName' :  "mozmilltest@irc.mozilla.invalid",
         'mail.smtp.defaultserver' :  "smtp1",
-        'mail.smtpserver.smtp1.hostname' :  "tinderbox",
+        'mail.smtpserver.smtp1.hostname' :  "tinderbox123",
         'mail.smtpserver.smtp1.username' :  "tinderbox",
         'mail.smtpservers' :  "smtp1",
         'messenger.account.account1.autoLogin' :  False,
