@@ -32,6 +32,7 @@
 #include "nsDirectoryServiceDefs.h"
 #include "nsMsgUtils.h"
 #include "nsILineInputStream.h"
+#include "nsIAsyncInputStream.h"
 #include "nsIMsgIncomingServer.h"
 #include "nsMimeTypes.h"
 #include "nsAlgorithm.h"
@@ -42,17 +43,8 @@
 
 using namespace mozilla;
 
-NS_IMPL_THREADSAFE_ADDREF(nsMsgProtocol)
-NS_IMPL_THREADSAFE_RELEASE(nsMsgProtocol)
-
-NS_INTERFACE_MAP_BEGIN(nsMsgProtocol)
-   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIChannel)
-   NS_INTERFACE_MAP_ENTRY(nsIStreamListener)
-   NS_INTERFACE_MAP_ENTRY(nsIRequestObserver)
-   NS_INTERFACE_MAP_ENTRY(nsIChannel)
-   NS_INTERFACE_MAP_ENTRY(nsIRequest)
-   NS_INTERFACE_MAP_ENTRY(nsITransportEventSink)
-NS_INTERFACE_MAP_END_THREADSAFE
+NS_IMPL_ISUPPORTS5(nsMsgProtocol, nsIChannel, nsIStreamListener,
+  nsIRequestObserver, nsIRequest, nsITransportEventSink)
 
 static PRUnichar *FormatStringWithHostNameByID(int32_t stringID, nsIMsgMailNewsUrl *msgUri);
 
@@ -527,10 +519,6 @@ nsresult nsMsgProtocol::LoadUrl(nsIURI * aURL, nsISupports * aConsumer)
 
         m_request = pump; // keep a reference to the pump so we can cancel it
 
-        // This helps when running URLs from the command line
-        rv = pump->SetLoadGroup(m_loadGroup);
-        NS_ENSURE_SUCCESS(rv, rv);
-
         // put us in a state where we are always notified of incoming data
         rv = pump->AsyncRead(this, urlSupports);
         NS_ASSERTION(NS_SUCCEEDED(rv), "AsyncRead failed");
@@ -1041,7 +1029,8 @@ nsresult nsMsgProtocol::DoNtlmStep2(nsCString &commandResponse, nsCString &respo
 class nsMsgProtocolStreamProvider : public nsIOutputStreamCallback
 {
 public:
-    NS_DECL_ISUPPORTS
+    // XXX this probably doesn't need to be threadsafe
+    NS_DECL_THREADSAFE_ISUPPORTS
 
     nsMsgProtocolStreamProvider() { }
     virtual ~nsMsgProtocolStreamProvider() {}
@@ -1117,14 +1106,13 @@ protected:
   nsCOMPtr<nsIInputStream>  mInStream;
 };
 
-// XXX this probably doesn't need to be threadsafe
-NS_IMPL_THREADSAFE_ISUPPORTS1(nsMsgProtocolStreamProvider,
+NS_IMPL_ISUPPORTS1(nsMsgProtocolStreamProvider,
                               nsIOutputStreamCallback)
 
 class nsMsgFilePostHelper : public nsIStreamListener
 {
 public:
-  NS_DECL_ISUPPORTS
+  NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_NSIREQUESTOBSERVER
   NS_DECL_NSISTREAMLISTENER
 
@@ -1139,13 +1127,7 @@ protected:
   nsCOMPtr<nsIWeakReference> mProtInstance;
 };
 
-NS_IMPL_THREADSAFE_ADDREF(nsMsgFilePostHelper)
-NS_IMPL_THREADSAFE_RELEASE(nsMsgFilePostHelper)
-
-NS_INTERFACE_MAP_BEGIN(nsMsgFilePostHelper)
-  NS_INTERFACE_MAP_ENTRY(nsIStreamListener)
-  NS_INTERFACE_MAP_ENTRY(nsIRequestObserver)
-NS_INTERFACE_MAP_END_THREADSAFE
+NS_IMPL_ISUPPORTS2(nsMsgFilePostHelper, nsIStreamListener, nsIRequestObserver)
 
 nsresult nsMsgFilePostHelper::Init(nsIOutputStream * aOutStream, nsMsgAsyncWriteProtocol * aProtInstance, nsIFile *aFileToPost)
 {
@@ -1549,8 +1531,6 @@ nsresult nsMsgAsyncWriteProtocol::SendData(const char * dataBuffer, bool aSuppre
   return mAsyncOutStream->AsyncWait(mProvider, 0, 0, mProviderThread);
 }
 
-#define MSGS_URL    "chrome://messenger/locale/messenger.properties"
-
 PRUnichar *FormatStringWithHostNameByID(int32_t stringID, nsIMsgMailNewsUrl *msgUri)
 {
   if (!msgUri)
