@@ -45,7 +45,7 @@ public:
   virtual ~nsMsgCompFields();
 
   /* this macro defines QueryInterface, AddRef and Release for this class */
-  NS_DECL_ISUPPORTS
+  NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_NSIMSGCOMPFIELDS
 
   typedef enum MsgHeaderID
@@ -60,7 +60,6 @@ public:
     MSG_NEWSGROUPS_HEADER_ID,
     MSG_FOLLOWUP_TO_HEADER_ID,
     MSG_SUBJECT_HEADER_ID,
-    MSG_ATTACHMENTS_HEADER_ID,
     MSG_ORGANIZATION_HEADER_ID,
     MSG_REFERENCES_HEADER_ID,
     MSG_OTHERRANDOMHEADERS_HEADER_ID,
@@ -70,10 +69,6 @@ public:
     MSG_MESSAGE_ID_HEADER_ID,
     MSG_X_TEMPLATE_HEADER_ID,
     MSG_DRAFT_ID_HEADER_ID,
-    MSG_TEMPORARY_FILES_HEADER_ID,
-    MSG_SENDERREPLY_HEADER_ID,
-    MSG_ALLREPLY_HEADER_ID,
-    MSG_LISTREPLY_HEADER_ID,
 
     MSG_MAX_HEADERS   //Must be the last one.
   } MsgHeaderID;
@@ -114,24 +109,12 @@ public:
   nsresult SetNewsgroups(const char *aValue) {return SetAsciiHeader(MSG_NEWSGROUPS_HEADER_ID, aValue);}
   const char* GetNewsgroups() {return GetAsciiHeader(MSG_NEWSGROUPS_HEADER_ID);}
 
-  const char* GetNewshost() {return GetAsciiHeader(MSG_NEWSPOSTURL_HEADER_ID);}
-
   nsresult SetFollowupTo(const char *aValue) {return SetAsciiHeader(MSG_FOLLOWUP_TO_HEADER_ID, aValue);}
   const char* GetFollowupTo() {return GetAsciiHeader(MSG_FOLLOWUP_TO_HEADER_ID);}
 
   nsresult SetSubject(const char *value) {return SetAsciiHeader(MSG_SUBJECT_HEADER_ID, value);}
   const char* GetSubject() {return GetAsciiHeader(MSG_SUBJECT_HEADER_ID);}
 
-  const char* GetAttachments() {
-    NS_ERROR("nsMsgCompFields::GetAttachments is not supported anymore, please use nsMsgCompFields::GetAttachmentsArray");
-    return GetAsciiHeader(MSG_ATTACHMENTS_HEADER_ID);
-    }
-
-  const char* GetTemporaryFiles() {
-    NS_ERROR("nsMsgCompFields::GetTemporaryFiles is not supported anymore, please use nsMsgCompFields::GetAttachmentsArray");
-    return GetAsciiHeader(MSG_TEMPORARY_FILES_HEADER_ID);
-    }
-
   nsresult SetOrganization(const char *value) {return SetAsciiHeader(MSG_ORGANIZATION_HEADER_ID, value);}
   const char* GetOrganization() {return GetAsciiHeader(MSG_ORGANIZATION_HEADER_ID);}
 
@@ -140,15 +123,6 @@ public:
   nsresult SetOtherRandomHeaders(const char *value) {return SetAsciiHeader(MSG_OTHERRANDOMHEADERS_HEADER_ID, value);}
   const char* GetOtherRandomHeaders() {return GetAsciiHeader(MSG_OTHERRANDOMHEADERS_HEADER_ID);}
 
-  nsresult SetSenderReply(const char *value) {return SetAsciiHeader(MSG_SENDERREPLY_HEADER_ID, value);}
-  const char* GetSenderReply() {return GetAsciiHeader(MSG_SENDERREPLY_HEADER_ID);}
-
-  nsresult SetAllReply(const char *value) {return SetAsciiHeader(MSG_ALLREPLY_HEADER_ID, value);}
-  const char* GetAllReply() {return GetAsciiHeader(MSG_ALLREPLY_HEADER_ID);}
-
-  nsresult SetListReply(const char *value) {return SetAsciiHeader(MSG_LISTREPLY_HEADER_ID, value);}
-  const char* GetListReply() {return GetAsciiHeader(MSG_LISTREPLY_HEADER_ID);}
-
   const char* GetNewspostUrl() {return GetAsciiHeader(MSG_NEWSPOSTURL_HEADER_ID);}
 
   const char* GetPriority() {return GetAsciiHeader(MSG_PRIORITY_HEADER_ID);}
