@@ -18,6 +18,14 @@ var FeedSubscriptions = {
   kCopyMode      : 4,
   kImportingOPML : 5,
 
+  get FOLDER_ACTIONS()
+  {
+    return Ci.nsIMsgFolderNotificationService.folderAdded |
+           Ci.nsIMsgFolderNotificationService.folderDeleted |
+           Ci.nsIMsgFolderNotificationService.folderRenamed |
+           Ci.nsIMsgFolderNotificationService.folderMoveCopyCompleted;
+  },
+
   onLoad: function ()
   {
     // Extract the folder argument.
@@ -37,14 +45,11 @@ var FeedSubscriptions = {
     {
       win.FeedFolderNotificationService = MailServices.mfn;
       win.FeedFolderNotificationService.addListener(this.FolderListener,
-        Ci.nsIMsgFolderNotificationService.folderAdded |
-        Ci.nsIMsgFolderNotificationService.folderDeleted |
-        Ci.nsIMsgFolderNotificationService.folderRenamed |
-        Ci.nsIMsgFolderNotificationService.folderMoveCopyCompleted);
+                                                    this.FOLDER_ACTIONS);
     }
   },
 
-  onUnload: function ()
+  onClose: function ()
   {
     let dismissDialog = true;
 
@@ -64,9 +69,14 @@ var FeedSubscriptions = {
 
     if (dismissDialog)
     {
+      FeedUtils.CANCEL_REQUESTED = true;
       let win = Services.wm.getMostRecentWindow("mail:3pane");
       if (win)
-        delete win.FeedFolderNotificationService;
+        {
+          win.FeedFolderNotificationService.removeListener(this.FolderListener,
+                                                           this.FOLDER_ACTIONS);
+          delete win.FeedFolderNotificationService;
+        }
     }
 
     return dismissDialog;
@@ -154,8 +164,19 @@ var FeedSubscriptions = {
     getCellProperties: function (aRow, aColumn) {
       let item = this.getItemAtIndex(aRow);
       let folder = item && item.folder ? item.folder : null;
+#ifdef MOZ_THUNDERBIRD
+      let properties = ["folderNameCol"];
+      let hasFeeds = folder ? FeedUtils.getFeedUrlsInFolder(folder) : false;
+      let prop = !folder ? "isFeed-true" :
+                 hasFeeds ? "isFeedFolder-true" :
+                 folder.isServer ? "serverType-rss isServer-true" : null;
+      if (prop)
+        properties.push(prop);
+      return properties.join(" ");
+#else
       return !folder ? "serverType-rss" :
-             folder.isServer ? "serverType=rss isServer-true" : "livemark";
+             folder.isServer ? "serverType-rss isServer-true" : "livemark";
+#endif
     },
 
     isContainer: function (aRow)
@@ -434,6 +455,18 @@ var FeedSubscriptions = {
       let rowCount = 0;
       let multiplier;
 
+      function addDescendants(aItem)
+      {
+        for (let i = 0; i < aItem.children.length; i++)
+        {
+          rowCount++;
+          let child = aItem.children[i];
+          rows.splice(aRow + rowCount, 0, child);
+          if (child.open)
+            addDescendants(child);
+        }
+      }
+
       if (item.open)
       {
         // Close the container.  Add up all subfolders and their descendants
@@ -454,18 +487,6 @@ var FeedSubscriptions = {
         // Open the container.  Restore the open state of all subfolder and
         // their descendants.
         multiplier = 1;
-        function addDescendants(aItem)
-        {
-          for (let i = 0; i < aItem.children.length; i++)
-          {
-            rowCount++;
-            let child = aItem.children[i];
-            rows.splice(aRow + rowCount, 0, child);
-            if (child.open)
-              addDescendants(child);
-          }
-        }
-
         addDescendants(item);
       }
 
@@ -1398,8 +1419,12 @@ var FeedSubscriptions = {
 
   mFeedDownloadCallback:
   {
+    mSubscribeMode: true,
     downloaded: function(feed, aErrorCode)
     {
+      // Offline check is done in the context of 3pane, return to the subscribe
+      // window once the modal prompt is dispatched.
+      window.focus();
       // Feed is null if our attempt to parse the feed failed.
       let message = "";
       let win = FeedSubscriptions;
@@ -1504,7 +1529,8 @@ var FeedSubscriptions = {
 
       win.mActionMode = null;
       win.clearStatusInfo();
-      win.updateStatusItem("statusText", message, aErrorCode);
+      let code = feed.url.startsWith("http") ? aErrorCode : null;
+      win.updateStatusItem("statusText", message, code);
     },
 
     // This gets called after the RSS parser finishes storing a feed item to
@@ -1513,6 +1539,7 @@ var FeedSubscriptions = {
     // corresponding to the total number of feed items to download.
     onFeedItemStored: function (feed, aCurrentFeedItems, aMaxFeedItems)
     {
+      window.focus();
       let message = FeedUtils.strings.formatStringFromName(
                       "subscribe-gettingFeedItems",
                       [aCurrentFeedItems, aMaxFeedItems], 2);
@@ -1954,7 +1981,7 @@ var FeedSubscriptions = {
   generateOutlineList: function(baseFolder, parent, indentLevel)
   {
     // Pretty printing.
-    let indentString = (new Array(indentLevel - 1)).join(" ");
+    let indentString = " ".repeat(indentLevel - 2);
 
     let feedOutline;
     let folderEnumerator = baseFolder.subFolders;
@@ -1995,7 +2022,7 @@ var FeedSubscriptions = {
   generateOutlineStruct: function(baseFolder, parent, indentLevel)
   {
     // Pretty printing.
-    function indentString(len) { return (new Array(len - 1)).join(" ") };
+    function indentString(len) { return " ".repeat(len - 2); };
 
     let folderOutline, feedOutline;
     let folderEnumerator = baseFolder.subFolders;
