@@ -52,11 +52,11 @@ static NS_DEFINE_CID(kMsgCompFieldsCID, NS_MSGCOMPFIELDS_CID);
 
 #ifdef IMPORT_DEBUG
 static const char *p_test_headers =
-"Received: from netppl.fi (IDENT:monitor@get.freebsd.because.microsoftsucks.net [209.3.31.115])\n\
- by mail4.sirius.com (8.9.1/8.9.1) with SMTP id PAA27232;\n\
+"Received: from netppl.invalid (IDENT:monitor@get.freebsd.because.microsoftsucks.invalid [209.3.31.115])\n\
+ by mail4.sirius.invalid (8.9.1/8.9.1) with SMTP id PAA27232;\n\
  Mon, 17 May 1999 15:27:43 -0700 (PDT)\n\
-Message-ID: <ikGD3jRTsKklU.Ggm2HmE2A1Jsqd0p@netppl.fi>\n\
-From: \"adsales@qualityservice.com\" <adsales@qualityservice.com>\n\
+Message-ID: <ikGD3jRTsKklU.Ggm2HmE2A1Jsqd0p@netppl.invalid>\n\
+From: \"adsales@qualityservice.invalid\" <adsales@qualityservice.invalid>\n\
 Subject: Re: Your College Diploma (36822)\n\
 Date: Mon, 17 May 1999 15:09:29 -0400 (EDT)\n\
 MIME-Version: 1.0\n\
@@ -69,8 +69,8 @@ static const char *p_test_body =
 "Hello world?\n\
 ";
 #else
-#define p_test_headers nsnull
-#define p_test_body nsnull
+#define p_test_headers nullptr
+#define p_test_body nullptr
 #endif
 
 #define kWhitespace "\b\t\r\n "
@@ -82,7 +82,7 @@ class CCompositionFile {
 public:
   // fifoBuffer is used for memory allocation optimization
   // convertCRs controls if we want to convert standalone CRs to CRLFs
-  CCompositionFile(nsIFile* aFile, void* fifoBuffer, PRUint32 fifoBufferSize, bool convertCRs=false);
+  CCompositionFile(nsIFile* aFile, void* fifoBuffer, uint32_t fifoBufferSize, bool convertCRs=false);
 
   operator bool() const { return m_fileSize && m_pInputStream; }
 
@@ -95,10 +95,10 @@ public:
 private:
   nsCOMPtr<nsIFile>  m_pFile;
   nsCOMPtr<nsIInputStream> m_pInputStream;
-  PRInt64 m_fileSize;
-  PRInt64 m_fileReadPos;
+  int64_t m_fileSize;
+  int64_t m_fileReadPos;
   char* m_fifoBuffer;
-  PRUint32 m_fifoBufferSize;
+  uint32_t m_fifoBufferSize;
   char* m_fifoBufferReadPos; // next character to read
   char* m_fifoBufferWrittenPos; // if we have read less than buffer size then this will show it
   bool m_convertCRs;
@@ -116,19 +116,19 @@ class OutlookSendListener : public nsIMsgSendListener
 public:
   OutlookSendListener() {
     m_done = false;
-    m_location = nsnull;
+    m_location = nullptr;
   }
 
   virtual ~OutlookSendListener() { NS_IF_RELEASE(m_location); }
 
   // nsISupports interface
-  NS_DECL_ISUPPORTS
+  NS_DECL_THREADSAFE_ISUPPORTS
 
-  /* void OnStartSending (in string aMsgID, in PRUint32 aMsgSize); */
-  NS_IMETHOD OnStartSending(const char *aMsgID, PRUint32 aMsgSize) {return NS_OK;}
+  /* void OnStartSending (in string aMsgID, in uint32_t aMsgSize); */
+  NS_IMETHOD OnStartSending(const char *aMsgID, uint32_t aMsgSize) {return NS_OK;}
 
-  /* void OnProgress (in string aMsgID, in PRUint32 aProgress, in PRUint32 aProgressMax); */
-  NS_IMETHOD OnProgress(const char *aMsgID, PRUint32 aProgress, PRUint32 aProgressMax) {return NS_OK;}
+  /* void OnProgress (in string aMsgID, in uint32_t aProgress, in uint32_t aProgressMax); */
+  NS_IMETHOD OnProgress(const char *aMsgID, uint32_t aProgress, uint32_t aProgressMax) {return NS_OK;}
 
   /* void OnStatus (in string aMsgID, in wstring aMsg); */
   NS_IMETHOD OnStatus(const char *aMsgID, const PRUnichar *aMsg) {return NS_OK;}
@@ -155,11 +155,11 @@ public:
   nsIFile * m_location;
 };
 
-NS_IMPL_THREADSAFE_ISUPPORTS1(OutlookSendListener, nsIMsgSendListener)
+NS_IMPL_ISUPPORTS1(OutlookSendListener, nsIMsgSendListener)
 
 nsresult OutlookSendListener::CreateSendListener(nsIMsgSendListener **ppListener)
 {
-  NS_PRECONDITION(ppListener != nsnull, "null ptr");
+  NS_PRECONDITION(ppListener != nullptr, "null ptr");
   NS_ENSURE_ARG_POINTER(ppListener);
 
   *ppListener = new OutlookSendListener();
@@ -176,18 +176,18 @@ nsresult OutlookSendListener::CreateSendListener(nsIMsgSendListener **ppListener
 /////////////////////////////////////////////////////////////////////////////////
 
 #define hackBeginA "begin"
-#define hackBeginW NS_L(hackBeginA)
+#define hackBeginW MOZ_UTF16(hackBeginA)
 #define hackEndA "\015\012end"
-#define hackEndW NS_L(hackEndA)
+#define hackEndW MOZ_UTF16(hackEndA)
 #define hackCRLFA "crlf"
-#define hackCRLFW NS_L(hackCRLFA)
+#define hackCRLFW MOZ_UTF16(hackCRLFA)
 #define hackAmpersandA "amp"
-#define hackAmpersandW NS_L(hackAmpersandA)
+#define hackAmpersandW MOZ_UTF16(hackAmpersandA)
 
 nsOutlookCompose::nsOutlookCompose()
 {
-  m_pListener = nsnull;
-  m_pMsgFields = nsnull;
+  m_pListener = nullptr;
+  m_pMsgFields = nullptr;
 
   m_optimizationBufferSize = 16*1024;
   m_optimizationBuffer = new char[m_optimizationBufferSize];
@@ -206,7 +206,7 @@ nsOutlookCompose::~nsOutlookCompose()
   delete[] m_optimizationBuffer;
 }
 
-nsIMsgIdentity * nsOutlookCompose::m_pIdentity = nsnull;
+nsIMsgIdentity * nsOutlookCompose::m_pIdentity = nullptr;
 
 nsresult nsOutlookCompose::CreateIdentity(void)
 {
@@ -324,8 +324,7 @@ nsresult nsOutlookCompose::ComposeTheMessage(nsMsgDeliverMode mode, CMapiMessage
                         m_pIdentity,                  // dummy identity
                         m_pMsgFields,                 // message fields
                         msg.BodyIsHtml() ? "text/html" : "text/plain",
-                        bodyA.get(),                  // body pointer
-                        bodyA.Length(),               // body length
+                        bodyA,                        // body pointer
                         mode == nsIMsgSend::nsMsgSaveAsDraft,
                         pAttach,                      // local attachments
                         embeddedObjects,
@@ -337,9 +336,9 @@ nsresult nsOutlookCompose::ComposeTheMessage(nsMsgDeliverMode mode, CMapiMessage
   }
   else {
     // wait for the listener to get done!
-    PRInt32 abortCnt = 0;
-    PRInt32 cnt = 0;
-    PRInt32 sleepCnt = 1;
+    int32_t abortCnt = 0;
+    int32_t cnt = 0;
+    int32_t sleepCnt = 1;
     while (!pListen->m_done && (abortCnt < kHungAbortCount)) {
       PR_Sleep(sleepCnt);
       cnt++;
@@ -386,13 +385,13 @@ nsresult nsOutlookCompose::CopyComposedMessage(nsIFile *pSrc,
   // Thus, the lines that look like "From ..." in the message must be escaped (see EscapeFromSpaceLine())
   int fromLineLen;
   const char* fromLine = origMsg.GetFromLine(fromLineLen);
-  PRUint32 written;
+  uint32_t written;
   nsresult rv = pDst->Write(fromLine, fromLineLen, &written);
 
   // Bug 219269
   // Write out the x-mozilla-status headers.
   char statusLine[50];
-  PRUint32 msgFlags = 0;
+  uint32_t msgFlags = 0;
   if (origMsg.IsRead())
     msgFlags |= nsMsgMessageFlags::Read;
   if (!origMsg.FullMessageDownloaded())
@@ -588,14 +587,14 @@ void nsOutlookCompose::UnhackBody(nsCString& txt)
 
   nsCString hackedString(hackBeginA);
   hackedString.Append(hackedPostfixA);
-  PRInt32 begin = txt.Find(hackedString);
+  int32_t begin = txt.Find(hackedString);
   if (begin == kNotFound)
     return;
   txt.Cut(begin, hackedString.Length());
 
   hackedString.Assign(hackEndA);
   hackedString.Append(hackedPostfixA);
-  PRInt32 end = MsgFind(txt, hackedString, false, begin);
+  int32_t end = MsgFind(txt, hackedString, false, begin);
   if (end == kNotFound)
     return; // ?
   txt.Cut(end, hackedString.Length());
@@ -638,7 +637,7 @@ bool nsOutlookCompose::GenerateHackSequence(const wchar_t* body, size_t origLen)
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 CCompositionFile::CCompositionFile(nsIFile* aFile, void* fifoBuffer,
-                                   PRUint32 fifoBufferSize, bool convertCRs)
+                                   uint32_t fifoBufferSize, bool convertCRs)
   : m_pFile(aFile), m_fileSize(0), m_fileReadPos(0),
     m_fifoBuffer(static_cast<char*>(fifoBuffer)),
     m_fifoBufferSize(fifoBufferSize),
@@ -665,13 +664,13 @@ nsresult CCompositionFile::EnsureHasDataInBuffer()
   if (m_fifoBufferReadPos < m_fifoBufferWrittenPos)
     return NS_OK;
   // Populate the buffer with new data!
-  PRUint32 count = m_fifoBufferSize;
+  uint32_t count = m_fifoBufferSize;
   if ((m_fileReadPos + count) > m_fileSize)
     count = m_fileSize - m_fileReadPos;
   if (!count)
     return NS_ERROR_FAILURE; // Isn't there a "No more data" error?
 
-  PRUint32 bytesRead = 0;
+  uint32_t bytesRead = 0;
   nsresult rv = m_pInputStream->Read(m_fifoBuffer, count, &bytesRead);
   NS_ENSURE_SUCCESS(rv, rv);
   if (!bytesRead || (bytesRead > count))
@@ -780,9 +779,9 @@ class dest_nsCString {
 public:
   dest_nsCString(nsCString& str) : m_str(str) { m_str.Truncate(); }
 #ifdef MOZILLA_INTERNAL_API
-  void SetCapacity(PRInt32 sz) { m_str.SetCapacity(sz); }
+  void SetCapacity(int32_t sz) { m_str.SetCapacity(sz); }
 #endif
-  nsresult Append(const char* buf, PRUint32 count) {
+  nsresult Append(const char* buf, uint32_t count) {
     m_str.Append(buf, count); return NS_OK; }
 private:
   nsCString& m_str;
@@ -792,11 +791,11 @@ class dest_Stream {
 public:
   dest_Stream(nsIOutputStream *dest) : m_stream(dest) {}
 #ifdef MOZILLA_INTERNAL_API
-  void SetCapacity(PRInt32) { /*do nothing*/ }
+  void SetCapacity(int32_t) { /*do nothing*/ }
 #endif
   // const_cast here is due to the poor design of the EscapeFromSpaceLine()
   // that requires a non-constant pointer while doesn't modify its data
-  nsresult Append(const char* buf, PRUint32 count) {
+  nsresult Append(const char* buf, uint32_t count) {
     return EscapeFromSpaceLine(m_stream, const_cast<char*>(buf), buf+count); }
 private:
   nsIOutputStream *m_stream;
