@@ -4,6 +4,7 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsCOMPtr.h"
+#include "nsICollation.h"
 #include "nsIDOMNode.h"
 #include "nsIRDFNode.h"
 #include "nsIRDFObserver.h"
@@ -18,6 +19,7 @@
 #include "nsAttrName.h"
 #include "rdf.h"
 #include "nsArrayUtils.h"
+#include "nsIURI.h"
 
 #include "nsContentTestNode.h"
 #include "nsRDFConInstanceTestNode.h"
@@ -31,6 +33,7 @@
 #include "nsXULTemplateResultSetRDF.h"
 #include "nsXULTemplateQueryProcessorRDF.h"
 #include "nsXULSortService.h"
+#include "nsIDocument.h"
 
 //----------------------------------------------------------------------
 
@@ -46,35 +49,36 @@ nsIRDFResource*           nsXULTemplateQueryProcessorRDF::kNC_BookmarkSeparator;
 nsIRDFResource*           nsXULTemplateQueryProcessorRDF::kRDF_type;
 
 NS_IMPL_CYCLE_COLLECTION_CLASS(nsXULTemplateQueryProcessorRDF)
+
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(nsXULTemplateQueryProcessorRDF)
     tmp->Done();
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
 static PLDHashOperator
 BindingDependenciesTraverser(nsISupports* key,
-                             nsCOMArray<nsXULTemplateResultRDF>* array,
+                             nsXULTemplateQueryProcessorRDF::ResultArray* array,
                              void* userArg)
 {
     nsCycleCollectionTraversalCallback *cb = 
         static_cast<nsCycleCollectionTraversalCallback*>(userArg);
 
-    PRInt32 i, count = array->Count();
+    int32_t i, count = array->Length();
     for (i = 0; i < count; ++i) {
-        cb->NoteXPCOMChild(array->ObjectAt(i));
+        cb->NoteXPCOMChild(array->ElementAt(i));
     }
 
     return PL_DHASH_NEXT;
 }
 
 static PLDHashOperator
-MemoryElementTraverser(const PRUint32& key,
+MemoryElementTraverser(const uint32_t& key,
                        nsCOMArray<nsXULTemplateResultRDF>* array,
                        void* userArg)
 {
     nsCycleCollectionTraversalCallback *cb = 
         static_cast<nsCycleCollectionTraversalCallback*>(userArg);
 
-    PRInt32 i, count = array->Count();
+    int32_t i, count = array->Count();
     for (i = 0; i < count; ++i) {
         cb->NoteXPCOMChild(array->ObjectAt(i));
     }
@@ -94,20 +98,14 @@ RuleToBindingTraverser(nsISupports* key, RDFBindingSet* binding, void* userArg)
 }
 
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(nsXULTemplateQueryProcessorRDF)
-    NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mDB)
-    NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mLastRef)
-    if (tmp->mBindingDependencies.IsInitialized()) {
-        tmp->mBindingDependencies.EnumerateRead(BindingDependenciesTraverser,
-                                                &cb);
-    }
-    if (tmp->mMemoryElementToResultMap.IsInitialized()) {
-        tmp->mMemoryElementToResultMap.EnumerateRead(MemoryElementTraverser,
-                                                     &cb);
-    }
-    if (tmp->mRuleToBindingsMap.IsInitialized()) {
-        tmp->mRuleToBindingsMap.EnumerateRead(RuleToBindingTraverser, &cb);
-    }
-    NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMARRAY(mQueries)
+    NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mDB)
+    NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mLastRef)
+    tmp->mBindingDependencies.EnumerateRead(BindingDependenciesTraverser,
+                                            &cb);
+    tmp->mMemoryElementToResultMap.EnumerateRead(MemoryElementTraverser,
+                                                 &cb);
+    tmp->mRuleToBindingsMap.EnumerateRead(RuleToBindingTraverser, &cb);
+    NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mQueries)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 NS_IMPL_CYCLE_COLLECTING_ADDREF(nsXULTemplateQueryProcessorRDF)
@@ -119,12 +117,12 @@ NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(nsXULTemplateQueryProcessorRDF)
 NS_INTERFACE_MAP_END
 
 nsXULTemplateQueryProcessorRDF::nsXULTemplateQueryProcessorRDF(void)
-    : mDB(nsnull),
-      mBuilder(nsnull),
+    : mDB(nullptr),
+      mBuilder(nullptr),
       mQueryProcessorRDFInited(false),
       mGenerationStarted(false),
       mUpdateBatchNest(0),
-      mSimpleRuleMemberTest(nsnull)
+      mSimpleRuleMemberTest(nullptr)
 {
     gRefCnt++;
 }
@@ -170,7 +168,7 @@ nsXULTemplateQueryProcessorRDF::InitGlobals()
                              &kRDF_type);
     }
 
-    return MemoryElement::Init() ? NS_OK : NS_ERROR_FAILURE;
+    return NS_OK;
 }
 
 //----------------------------------------------------------------------
@@ -190,7 +188,7 @@ nsXULTemplateQueryProcessorRDF::GetDatasource(nsIArray* aDataSources,
     nsCOMPtr<nsIContent> root = do_QueryInterface(aRootNode);
     nsresult rv;
 
-    *aResult = nsnull;
+    *aResult = nullptr;
     *aShouldDelayBuilding = false;
 
     NS_ENSURE_TRUE(root, NS_ERROR_UNEXPECTED);
@@ -233,7 +231,7 @@ nsXULTemplateQueryProcessorRDF::GetDatasource(nsIArray* aDataSources,
         NS_ENSURE_SUCCESS(rv, rv);
     }
 
-    PRUint32 length, index;
+    uint32_t length, index;
     rv = aDataSources->GetLength(&length);
     NS_ENSURE_SUCCESS(rv,rv);
 
@@ -244,7 +242,7 @@ nsXULTemplateQueryProcessorRDF::GetDatasource(nsIArray* aDataSources,
             continue;
 
         nsCOMPtr<nsIRDFDataSource> ds;
-        nsCAutoString uristrC;
+        nsAutoCString uristrC;
         uri->GetSpec(uristrC);
 
         rv = gRDFService->GetDataSource(uristrC.get(), getter_AddRefs(ds));
@@ -254,7 +252,7 @@ nsXULTemplateQueryProcessorRDF::GetDatasource(nsIArray* aDataSources,
             // be accessible for any number of reasons, including
             // security, a bad URL, etc.
   #ifdef DEBUG
-            nsCAutoString msg;
+            nsAutoCString msg;
             msg.Append("unable to load datasource '");
             msg.Append(uristrC);
             msg.Append('\'');
@@ -302,13 +300,6 @@ nsXULTemplateQueryProcessorRDF::InitializeForBuilding(nsISupports* aDatasource,
         if (NS_FAILED(rv))
             return rv;
 
-        if (!mMemoryElementToResultMap.IsInitialized())
-            mMemoryElementToResultMap.Init();
-        if (!mBindingDependencies.IsInitialized())
-            mBindingDependencies.Init();
-        if (!mRuleToBindingsMap.IsInitialized())
-            mRuleToBindingsMap.Init();
-
         mQueryProcessorRDFInited = true;
     }
 
@@ -337,10 +328,10 @@ nsXULTemplateQueryProcessorRDF::Done()
     if (mDB)
         mDB->RemoveObserver(this);
 
-    mDB = nsnull;
-    mBuilder = nsnull;
-    mRefVariable = nsnull;
-    mLastRef = nsnull;
+    mDB = nullptr;
+    mBuilder = nullptr;
+    mRefVariable = nullptr;
+    mLastRef = nullptr;
 
     mGenerationStarted = false;
     mUpdateBatchNest = 0;
@@ -355,7 +346,7 @@ nsXULTemplateQueryProcessorRDF::Done()
     mRDFTests.Clear();
     mQueries.Clear();
 
-    mSimpleRuleMemberTest = nsnull;
+    mSimpleRuleMemberTest = nullptr;
 
     mBindingDependencies.Clear();
 
@@ -387,7 +378,7 @@ nsXULTemplateQueryProcessorRDF::CompileQuery(nsIXULTemplateBuilder* aBuilder,
         query->mMemberVariable = aMemberVariable;
 
     nsresult rv;
-    TestNode *lastnode = nsnull;
+    TestNode *lastnode = nullptr;
 
     nsCOMPtr<nsIContent> content = do_QueryInterface(aQueryNode);
 
@@ -433,9 +424,7 @@ nsXULTemplateQueryProcessorRDF::CompileQuery(nsIXULTemplateBuilder* aBuilder,
     if (NS_FAILED(rv))
         return rv;
 
-    rv = mQueries.AppendObject(query);
-    if (NS_FAILED(rv))
-        return rv;
+    mQueries.AppendElement(query);
 
     *_retval = query;
     NS_ADDREF(*_retval);
@@ -459,7 +448,7 @@ nsXULTemplateQueryProcessorRDF::GenerateResults(nsISupports* aDatasource,
     // non-scriptable nsITemplateRDFQuery
     nsRDFQuery* query = static_cast<nsRDFQuery *>(aQuery);
 
-    *aResults = nsnull;
+    *aResults = nullptr;
 
     nsCOMPtr<nsISimpleEnumerator> results;
 
@@ -472,8 +461,8 @@ nsXULTemplateQueryProcessorRDF::GenerateResults(nsISupports* aDatasource,
         }
         else {
             // clear the cached results
-            PRInt32 count = mQueries.Count();
-            for (PRInt32 r = 0; r < count; r++) {
+            int32_t count = mQueries.Length();
+            for (int32_t r = 0; r < count; r++) {
                 mQueries[r]->ClearCachedResults();
             }
         }
@@ -545,7 +534,7 @@ nsXULTemplateQueryProcessorRDF::GenerateResults(nsISupports* aDatasource,
 
     if (! results) {
         // no results were found so create an empty set
-        results = new nsXULTemplateResultSetRDF(this, query, nsnull);
+        results = new nsXULTemplateResultSetRDF(this, query, nullptr);
         if (! results)
             return NS_ERROR_OUT_OF_MEMORY;
     }
@@ -610,8 +599,8 @@ NS_IMETHODIMP
 nsXULTemplateQueryProcessorRDF::CompareResults(nsIXULTemplateResult* aLeft,
                                                nsIXULTemplateResult* aRight,
                                                nsIAtom* aVar,
-                                               PRUint32 aSortHints,
-                                               PRInt32* aResult)
+                                               uint32_t aSortHints,
+                                               int32_t* aResult)
 {
     NS_ENSURE_ARG_POINTER(aLeft);
     NS_ENSURE_ARG_POINTER(aRight);
@@ -622,8 +611,8 @@ nsXULTemplateQueryProcessorRDF::CompareResults(nsIXULTemplateResult* aLeft,
     // order. If there is no container, just sort them arbitrarily.
     if (!aVar) {
         // if a result has a negative index, just sort it first
-        PRInt32 leftindex = GetContainerIndexOf(aLeft);
-        PRInt32 rightindex = GetContainerIndexOf(aRight);
+        int32_t leftindex = GetContainerIndexOf(aLeft);
+        int32_t rightindex = GetContainerIndexOf(aRight);
         *aResult = leftindex == rightindex ? 0 :
                    leftindex > rightindex ? 1 :
                    -1;
@@ -693,12 +682,10 @@ nsXULTemplateQueryProcessorRDF::CompareResults(nsIXULTemplateResult* aLeft,
                 l->GetValue(&ldate);
                 r->GetValue(&rdate);
 
-                PRInt64 delta;
-                LL_SUB(delta, ldate, rdate);
-
-                if (LL_IS_ZERO(delta))
+                int64_t delta = ldate - rdate;
+                if (delta == 0)
                     *aResult = 0;
-                else if (LL_GE_ZERO(delta))
+                else if (delta >= 0)
                     *aResult = 1;
                 else
                     *aResult = -1;
@@ -714,7 +701,7 @@ nsXULTemplateQueryProcessorRDF::CompareResults(nsIXULTemplateResult* aLeft,
         if (l) {
             nsCOMPtr<nsIRDFInt> r = do_QueryInterface(rightNode);
             if (r) {
-                PRInt32 lval, rval;
+                int32_t lval, rval;
                 l->GetValue(&lval);
                 r->GetValue(&rval);
 
@@ -733,8 +720,8 @@ nsXULTemplateQueryProcessorRDF::CompareResults(nsIXULTemplateResult* aLeft,
         if (l) {
             nsCOMPtr<nsIRDFBlob> r = do_QueryInterface(rightNode);
             if (r) {
-                const PRUint8 *lval, *rval;
-                PRInt32 llen, rlen;
+                const uint8_t *lval, *rval;
+                int32_t llen, rlen;
                 l->GetValue(&lval);
                 l->GetLength(&llen);
                 r->GetValue(&rval);
@@ -771,7 +758,7 @@ nsXULTemplateQueryProcessorRDF::OnAssert(nsIRDFDataSource* aDataSource,
     LOG("onassert", aSource, aProperty, aTarget);
 
     Propagate(aSource, aProperty, aTarget);
-    SynchronizeAll(aSource, aProperty, nsnull, aTarget);
+    SynchronizeAll(aSource, aProperty, nullptr, aTarget);
     return NS_OK;
 }
 
@@ -793,7 +780,7 @@ nsXULTemplateQueryProcessorRDF::OnUnassert(nsIRDFDataSource* aDataSource,
     LOG("onunassert", aSource, aProperty, aTarget);
 
     Retract(aSource, aProperty, aTarget);
-    SynchronizeAll(aSource, aProperty, aTarget, nsnull);
+    SynchronizeAll(aSource, aProperty, aTarget, nullptr);
     return NS_OK;
 }
 
@@ -1013,13 +1000,13 @@ nsXULTemplateQueryProcessorRDF::SynchronizeAll(nsIRDFResource* aSource,
 
     // Get all the matches whose assignments are currently supported
     // by aSource and aProperty: we'll need to recompute them.
-    nsCOMArray<nsXULTemplateResultRDF>* results;
+    ResultArray* results;
     if (!mBindingDependencies.Get(aSource, &results) || !mBuilder)
         return NS_OK;
 
-    PRUint32 length = results->Count();
+    uint32_t length = results->Length();
 
-    for (PRUint32 r = 0; r < length; r++) {
+    for (uint32_t r = 0; r < length; r++) {
         nsXULTemplateResultRDF* result = (*results)[r];
         if (result) {
             // synchronize the result's bindings and then update the builder
@@ -1067,7 +1054,7 @@ nsXULTemplateQueryProcessorRDF::Log(const char* aOperation,
         if (NS_FAILED(rv))
             return rv;
 
-        nsCAutoString targetstrC;
+        nsAutoCString targetstrC;
         targetstrC.AssignWithConversion(targetStr);
         PR_LOG(gXULTemplateLog, PR_LOG_DEBUG,
                ("                        --[%s]-->[%s]",
@@ -1168,8 +1155,8 @@ nsXULTemplateQueryProcessorRDF::ComputeContainmentProperties(nsIDOMNode* aRootNo
     nsAutoString containment;
     content->GetAttr(kNameSpaceID_None, nsGkAtoms::containment, containment);
 
-    PRUint32 len = containment.Length();
-    PRUint32 offset = 0;
+    uint32_t len = containment.Length();
+    uint32_t offset = 0;
     while (offset < len) {
         while (offset < len && nsCRT::IsAsciiSpace(containment[offset]))
             ++offset;
@@ -1177,7 +1164,7 @@ nsXULTemplateQueryProcessorRDF::ComputeContainmentProperties(nsIDOMNode* aRootNo
         if (offset >= len)
             break;
 
-        PRUint32 end = offset;
+        uint32_t end = offset;
         while (end < len && !nsCRT::IsAsciiSpace(containment[end]))
             ++end;
 
@@ -1258,7 +1245,7 @@ nsXULTemplateQueryProcessorRDF::CompileExtendedQuery(nsRDFQuery* aQuery,
             continue;
         }
 
-        TestNode* testnode = nsnull;
+        TestNode* testnode = nullptr;
         nsresult rv = CompileQueryChild(condition->Tag(), aQuery, condition,
                                         prevnode, &testnode);
         if (NS_FAILED(rv))
@@ -1298,7 +1285,7 @@ nsXULTemplateQueryProcessorRDF::CompileQueryChild(nsIAtom* aTag,
         nsAutoString tagstr;
         aTag->ToString(tagstr);
 
-        nsCAutoString tagstrC;
+        nsAutoCString tagstrC;
         tagstrC.AssignWithConversion(tagstr);
         PR_LOG(gXULTemplateLog, PR_LOG_ALWAYS,
                ("xultemplate[%p] unrecognized condition test <%s>",
@@ -1317,12 +1304,12 @@ nsXULTemplateQueryProcessorRDF::ParseLiteral(const nsString& aParseType,
                                              nsIRDFNode** aResult)
 {
     nsresult rv = NS_OK;
-    *aResult = nsnull;
+    *aResult = nullptr;
 
     if (aParseType.EqualsLiteral(PARSE_TYPE_INTEGER)) {
         nsCOMPtr<nsIRDFInt> intLiteral;
-        PRInt32 errorCode;
-        PRInt32 intValue = aValue.ToInteger(&errorCode);
+        nsresult errorCode;
+        int32_t intValue = aValue.ToInteger(&errorCode);
         if (NS_FAILED(errorCode))
             return NS_ERROR_FAILURE;
         rv = gRDFService->GetIntLiteral(intValue, getter_AddRefs(intLiteral));
@@ -1409,7 +1396,7 @@ nsXULTemplateQueryProcessorRDF::CompileTripleCondition(nsRDFQuery* aQuery,
             return rv;
     }
 
-    nsRDFPropertyTestNode* testnode = nsnull;
+    nsRDFPropertyTestNode* testnode = nullptr;
 
     if (svar && ovar) {
         testnode = new nsRDFPropertyTestNode(aParentNode, this, svar, pres, ovar);
@@ -1576,7 +1563,7 @@ nsXULTemplateQueryProcessorRDF::CompileSimpleQuery(nsRDFQuery* aQuery,
 
     // Add constraints for the LHS
     const nsAttrName* name;
-    for (PRUint32 i = 0; (name = aQueryElement->GetAttrNameAt(i)); ++i) {
+    for (uint32_t i = 0; (name = aQueryElement->GetAttrNameAt(i)); ++i) {
         // Note: some attributes must be skipped on XUL template query subtree
 
         // never compare against rdf:property, rdf:instanceOf, {}:id or {}:parsetype attribute
@@ -1587,7 +1574,7 @@ nsXULTemplateQueryProcessorRDF::CompileSimpleQuery(nsRDFQuery* aQuery,
             continue;
         }
 
-        PRInt32 attrNameSpaceID = name->NamespaceID();
+        int32_t attrNameSpaceID = name->NamespaceID();
         if (attrNameSpaceID == kNameSpaceID_XMLNS)
           continue;
         nsIAtom* attr = name->LocalName();
@@ -1595,7 +1582,7 @@ nsXULTemplateQueryProcessorRDF::CompileSimpleQuery(nsRDFQuery* aQuery,
         nsAutoString value;
         aQueryElement->GetAttr(attrNameSpaceID, attr, value);
 
-        TestNode* testnode = nsnull;
+        TestNode* testnode = nullptr;
 
         if (name->Equals(nsGkAtoms::iscontainer, kNameSpaceID_None) ||
             name->Equals(nsGkAtoms::isempty, kNameSpaceID_None)) {
@@ -1609,7 +1596,7 @@ nsXULTemplateQueryProcessorRDF::CompileSimpleQuery(nsRDFQuery* aQuery,
                 nsRDFConInstanceTestNode::eDontCare;
 
             static nsIContent::AttrValuesArray strings[] =
-              {&nsGkAtoms::_true, &nsGkAtoms::_false, nsnull};
+              {&nsGkAtoms::_true, &nsGkAtoms::_false, nullptr};
             switch (aQueryElement->FindAttrValueIn(kNameSpaceID_None,
                                                    nsGkAtoms::iscontainer,
                                                    strings, eCaseMatters)) {
@@ -1712,38 +1699,32 @@ nsXULTemplateQueryProcessorRDF::GetBindingsForRule(nsIDOMNode* aRuleNode)
     return mRuleToBindingsMap.GetWeak(aRuleNode);
 }
 
-nsresult
+void
 nsXULTemplateQueryProcessorRDF::AddBindingDependency(nsXULTemplateResultRDF* aResult,
                                                      nsIRDFResource* aResource)
 {
-    nsCOMArray<nsXULTemplateResultRDF>* arr;
+    ResultArray* arr;
     if (!mBindingDependencies.Get(aResource, &arr)) {
-        arr = new nsCOMArray<nsXULTemplateResultRDF>();
-        if (!arr)
-            return NS_ERROR_OUT_OF_MEMORY;
+        arr = new ResultArray();
 
         mBindingDependencies.Put(aResource, arr);
     }
 
-    PRInt32 index = arr->IndexOf(aResult);
+    int32_t index = arr->IndexOf(aResult);
     if (index == -1)
-        return arr->AppendObject(aResult);
-
-    return NS_OK;
+        arr->AppendElement(aResult);
 }
 
-nsresult
+void
 nsXULTemplateQueryProcessorRDF::RemoveBindingDependency(nsXULTemplateResultRDF* aResult,
                                                         nsIRDFResource* aResource)
 {
-    nsCOMArray<nsXULTemplateResultRDF>* arr;
+    ResultArray* arr;
     if (mBindingDependencies.Get(aResource, &arr)) {
-        PRInt32 index = arr->IndexOf(aResult);
+        int32_t index = arr->IndexOf(aResult);
         if (index >= 0)
-            return arr->RemoveObjectAt(index);
+            arr->RemoveElementAt(index);
     }
-
-    return NS_OK;
 }
 
 
@@ -1787,11 +1768,11 @@ nsXULTemplateQueryProcessorRDF::RemoveMemoryElements(const Instantiation& aInst,
 
         nsCOMArray<nsXULTemplateResultRDF>* arr;
         if (mMemoryElementToResultMap.Get(hash, &arr)) {
-            PRInt32 index = arr->IndexOf(aResult);
+            int32_t index = arr->IndexOf(aResult);
             if (index >= 0)
                 arr->RemoveObjectAt(index);
 
-            PRUint32 length = arr->Count();
+            uint32_t length = arr->Count();
             if (! length)
                 mMemoryElementToResultMap.Remove(hash);
         }
@@ -1813,9 +1794,9 @@ nsXULTemplateQueryProcessorRDF::RetractElement(const MemoryElement& aMemoryEleme
 
     nsCOMArray<nsXULTemplateResultRDF>* arr;
     if (mMemoryElementToResultMap.Get(hash, &arr)) {
-        PRUint32 length = arr->Count();
+        uint32_t length = arr->Count();
 
-        for (PRInt32 r = length - 1; r >= 0; r--) {
+        for (int32_t r = length - 1; r >= 0; r--) {
             nsXULTemplateResultRDF* result = (*arr)[r];
             if (result) {
                 // because the memory elements are hashed by an integer,
@@ -1832,13 +1813,13 @@ nsXULTemplateQueryProcessorRDF::RetractElement(const MemoryElement& aMemoryEleme
                     }
 
                     // a call to RemoveMemoryElements may have removed it
-                    if (!mMemoryElementToResultMap.Get(hash, nsnull))
+                    if (!mMemoryElementToResultMap.Get(hash, nullptr))
                         return;
 
                     // the array should have been reduced by one, but check
                     // just to make sure
-                    PRUint32 newlength = arr->Count();
-                    if (r > (PRInt32)newlength)
+                    uint32_t newlength = arr->Count();
+                    if (r > (int32_t)newlength)
                         r = newlength;
                 }
             }
@@ -1850,7 +1831,7 @@ nsXULTemplateQueryProcessorRDF::RetractElement(const MemoryElement& aMemoryEleme
     }
 }
 
-PRInt32
+int32_t
 nsXULTemplateQueryProcessorRDF::GetContainerIndexOf(nsIXULTemplateResult* aResult)
 {
     // get the reference variable and look up the container in the result
@@ -1870,7 +1851,7 @@ nsXULTemplateQueryProcessorRDF::GetContainerIndexOf(nsIXULTemplateResult* aResul
             nsCOMPtr<nsIRDFResource> resource;
             aResult->GetResource(getter_AddRefs(resource));
             if (resource) {
-                PRInt32 index;
+                int32_t index;
                 gRDFContainerUtils->IndexOf(mDB, container, resource, &index);
                 return index;
             }
