@@ -10,20 +10,35 @@
 
 #include "nsIDOMWindow.h"
 
-#include "nsIDOMLocation.h"
-#include "nsIDOMXULCommandDispatcher.h"
-#include "nsIDOMElement.h"
-#include "nsIDOMEventTarget.h"
-#include "nsIDOMDocument.h"
 #include "nsCOMPtr.h"
-#include "nsIURI.h"
+#include "nsAutoPtr.h"
+#include "nsTArray.h"
+#include "mozilla/dom/EventTarget.h"
+#include "js/TypeDecls.h"
 
 #define DOM_WINDOW_DESTROYED_TOPIC "dom-window-destroyed"
 #define DOM_WINDOW_FROZEN_TOPIC "dom-window-frozen"
 #define DOM_WINDOW_THAWED_TOPIC "dom-window-thawed"
 
+class nsIArray;
+class nsIContent;
+class nsIDocShell;
+class nsIDocument;
 class nsIIdleObserver;
 class nsIPrincipal;
+class nsIScriptTimeoutHandler;
+class nsIURI;
+class nsPerformance;
+class nsPIWindowRoot;
+class nsXBLPrototypeHandler;
+struct nsTimeout;
+
+namespace mozilla {
+namespace dom {
+class AudioContext;
+class Element;
+}
+}
 
 // Popup control state enum. The values in this enum must go from most
 // permissive to least permissive so that it's safe to push state in
@@ -37,19 +52,16 @@ enum PopupControlState {
   openOverridden    // disallow window open
 };
 
-class nsIDocShell;
-class nsIContent;
-class nsIDocument;
-class nsIScriptTimeoutHandler;
-struct nsTimeout;
-template <class> class nsScriptObjectHolder;
-class nsXBLPrototypeHandler;
-class nsIArray;
-class nsPIWindowRoot;
+enum UIStateChangeType
+{
+  UIStateChangeType_NoChange,
+  UIStateChangeType_Set,
+  UIStateChangeType_Clear
+};
 
 #define NS_PIDOMWINDOW_IID \
-{ 0x54fd92bd, 0xda33, 0x4451, \
-  { 0x8f, 0xb5, 0x11, 0x20, 0x5c, 0x03, 0xce, 0xaa } }
+{ 0x4f4eadf9, 0xe795, 0x48e5, \
+  { 0x89, 0x4b, 0x04, 0x40, 0xb2, 0x5d, 0xa6, 0xfa } }
 
 class nsPIDOMWindow : public nsIDOMWindowInternal
 {
@@ -94,14 +106,14 @@ public:
     return mIsBackground;
   }
 
-  nsIDOMEventTarget* GetChromeEventHandler() const
+  mozilla::dom::EventTarget* GetChromeEventHandler() const
   {
     return mChromeEventHandler;
   }
 
-  virtual void SetChromeEventHandler(nsIDOMEventTarget* aChromeEventHandler) = 0;
+  virtual void SetChromeEventHandler(mozilla::dom::EventTarget* aChromeEventHandler) = 0;
 
-  nsIDOMEventTarget* GetParentTarget()
+  mozilla::dom::EventTarget* GetParentTarget()
   {
     if (!mParentTarget) {
       UpdateParentTarget();
@@ -162,15 +174,12 @@ public:
   virtual void MaybeUpdateTouchState() {}
   virtual void UpdateTouchState() {}
 
-  // GetExtantDocument provides a backdoor to the DOM GetDocument accessor
-  nsIDOMDocument* GetExtantDocument() const
-  {
-    return mDocument;
-  }
   nsIDocument* GetExtantDoc() const
   {
     return mDoc;
   }
+  nsIURI* GetDocumentURI() const;
+  nsIURI* GetDocBaseURI() const;
 
   nsIDocument* GetDoc()
   {
@@ -189,34 +198,8 @@ public:
   // Internal getter/setter for the frame element, this version of the
   // getter crosses chrome boundaries whereas the public scriptable
   // one doesn't for security reasons.
-  nsIDOMElement* GetFrameElementInternal() const
-  {
-    if (mOuterWindow) {
-      return mOuterWindow->GetFrameElementInternal();
-    }
-
-    NS_ASSERTION(!IsInnerWindow(),
-                 "GetFrameElementInternal() called on orphan inner window");
-
-    return mFrameElement;
-  }
-
-  void SetFrameElementInternal(nsIDOMElement *aFrameElement)
-  {
-    if (IsOuterWindow()) {
-      mFrameElement = aFrameElement;
-
-      return;
-    }
-
-    if (!mOuterWindow) {
-      NS_ERROR("frameElement set on inner window with no outer!");
-
-      return;
-    }
-
-    mOuterWindow->SetFrameElementInternal(aFrameElement);
-  }
+  mozilla::dom::Element* GetFrameElementInternal() const;
+  void SetFrameElementInternal(mozilla::dom::Element* aFrameElement);
 
   bool IsLoadingOrRunningTimeout() const
   {
@@ -332,9 +315,8 @@ public:
   nsPIDOMWindow *EnsureInnerWindow()
   {
     NS_ASSERTION(IsOuterWindow(), "EnsureInnerWindow called on inner window");
-    // GetDocument forces inner window creation if there isn't one already
-    nsCOMPtr<nsIDOMDocument> doc;
-    GetDocument(getter_AddRefs(doc));
+    // GetDoc forces inner window creation if there isn't one already
+    GetDoc();
     return GetCurrentInnerWindow();
   }
 
@@ -343,6 +325,15 @@ public:
     return mIsInnerWindow;
   }
 
+
+  bool HasActiveDocument()
+  {
+    return GetOuterWindow() &&
+      (GetOuterWindow()->GetCurrentInnerWindow() == this ||
+       (GetOuterWindow()->GetCurrentInnerWindow() &&
+        GetOuterWindow()->GetCurrentInnerWindow()->GetDoc() == mDoc));
+  }
+
   bool IsOuterWindow() const
   {
     return !IsInnerWindow();
@@ -405,8 +396,8 @@ public:
    * Callback for notifying a window about a modal dialog being
    * opened/closed with the window as a parent.
    */
-  virtual nsIDOMWindow *EnterModalState() = 0;
-  virtual void LeaveModalState(nsIDOMWindow *) = 0;
+  virtual void EnterModalState() = 0;
+  virtual void LeaveModalState() = 0;
 
   virtual bool CanClose() = 0;
   virtual nsresult ForceClose() = 0;
@@ -482,7 +473,7 @@ public:
 
   virtual JSObject* GetCachedXBLPrototypeHandler(nsXBLPrototypeHandler* aKey) = 0;
   virtual void CacheXBLPrototypeHandler(nsXBLPrototypeHandler* aKey,
-                                        nsScriptObjectHolder<JSObject>& aHandler) = 0;
+                                        JS::Handle<JSObject*> aHandler) = 0;
 
   /*
    * Get and set the currently focused element within the document. If
@@ -588,14 +579,22 @@ public:
   virtual void DisableNetworkEvent(uint32_t aType) = 0;
 #endif // MOZ_B2G
 
+  /**
+   * Tell this window that there is an observer for gamepad input
+   */
+  virtual void SetHasGamepadEventListener(bool aHasGamepad = true) = 0;
+
   /**
    * Set a arguments for this window. This will be set on the window
    * right away (if there's an existing document) and it will also be
-   * installed on the window when the next document is loaded. Each
-   * language impl is responsible for converting to an array of args
-   * as appropriate for that language.
+   * installed on the window when the next document is loaded.
+   *
+   * This function serves double-duty for passing both |arguments| and
+   * |dialogArguments| back from nsWindowWatcher to nsGlobalWindow. For the
+   * latter, the array is an array of length 0 whose only element is a
+   * DialogArgumentsHolder representing the JS value passed to showModalDialog.
    */
-  virtual nsresult SetArguments(nsIArray *aArguments, nsIPrincipal *aOrigin) = 0;
+  virtual nsresult SetArguments(nsIArray *aArguments) = 0;
 
   /**
    * NOTE! This function *will* be called on multiple threads so the
@@ -628,6 +627,30 @@ public:
   OpenNoNavigate(const nsAString& aUrl, const nsAString& aName,
                  const nsAString& aOptions, nsIDOMWindow **_retval) = 0;
 
+  void AddAudioContext(mozilla::dom::AudioContext* aAudioContext);
+  void RemoveAudioContext(mozilla::dom::AudioContext* aAudioContext);
+  void MuteAudioContexts();
+  void UnmuteAudioContexts();
+
+  // Given an inner window, return its outer if the inner is the current inner.
+  // Otherwise (argument null or not an inner or not current) return null.
+  static nsPIDOMWindow* GetOuterFromCurrentInner(nsPIDOMWindow* aInner)
+  {
+    if (!aInner) {
+      return nullptr;
+    }
+
+    nsPIDOMWindow* outer = aInner->GetOuterWindow();
+    if (!outer || outer->GetCurrentInnerWindow() != aInner) {
+      return nullptr;
+    }
+
+    return outer;
+  }
+
+  // WebIDL-ish APIs
+  nsPerformance* GetPerformance();
+
 protected:
   // The nsPIDOMWindow constructor. The aOuterWindow argument should
   // be null if and only if the created window itself is an outer
@@ -637,7 +660,7 @@ protected:
 
   ~nsPIDOMWindow();
 
-  void SetChromeEventHandlerInternal(nsIDOMEventTarget* aChromeEventHandler) {
+  void SetChromeEventHandlerInternal(mozilla::dom::EventTarget* aChromeEventHandler) {
     mChromeEventHandler = aChromeEventHandler;
     // mParentTarget will be set when the next event is dispatched.
     mParentTarget = nullptr;
@@ -645,19 +668,27 @@ protected:
 
   virtual void UpdateParentTarget() = 0;
 
+  // Helper for creating performance objects.
+  void CreatePerformanceObjectIfNeeded();
+
   // These two variables are special in that they're set to the same
   // value on both the outer window and the current inner window. Make
   // sure you keep them in sync!
-  nsCOMPtr<nsIDOMEventTarget> mChromeEventHandler; // strong
-  nsCOMPtr<nsIDOMDocument> mDocument; // strong
-  nsCOMPtr<nsIDocument> mDoc; // strong, for fast access
+  nsCOMPtr<mozilla::dom::EventTarget> mChromeEventHandler; // strong
+  nsCOMPtr<nsIDocument> mDoc; // strong
+  // Cache the URI when mDoc is cleared.
+  nsCOMPtr<nsIURI> mDocumentURI; // strong
+  nsCOMPtr<nsIURI> mDocBaseURI; // strong
 
-  nsCOMPtr<nsIDOMEventTarget> mParentTarget; // strong
+  nsCOMPtr<mozilla::dom::EventTarget> mParentTarget; // strong
 
   // These members are only used on outer windows.
-  nsCOMPtr<nsIDOMElement> mFrameElement;
+  nsCOMPtr<mozilla::dom::Element> mFrameElement;
   nsIDocShell           *mDocShell;  // Weak Reference
 
+  // mPerformance is only used on inner windows.
+  nsRefPtr<nsPerformance>       mPerformance;
+
   uint32_t               mModalStateDepth;
 
   // These variables are only used on inner windows.
@@ -693,6 +724,9 @@ protected:
   // window is active
   nsCOMPtr<nsIContent> mFocusedNode;
 
+  // The AudioContexts created for the current document, if any.
+  nsTArray<mozilla::dom::AudioContext*> mAudioContexts; // Weak
+
   // A unique (as long as our 64-bit counter doesn't roll over) id for
   // this window.
   uint64_t mWindowID;
@@ -705,7 +739,7 @@ protected:
 
 NS_DEFINE_STATIC_IID_ACCESSOR(nsPIDOMWindow, NS_PIDOMWINDOW_IID)
 
-#ifdef _IMPL_NS_LAYOUT
+#ifdef MOZILLA_INTERNAL_API
 PopupControlState
 PushPopupControlState(PopupControlState aState, bool aForce);
 
@@ -726,7 +760,7 @@ PopPopupControlState(PopupControlState aState);
 class NS_AUTO_POPUP_STATE_PUSHER
 {
 public:
-#ifdef _IMPL_NS_LAYOUT
+#ifdef MOZILLA_INTERNAL_API
   NS_AUTO_POPUP_STATE_PUSHER(PopupControlState aState, bool aForce = false)
     : mOldState(::PushPopupControlState(aState, aForce))
   {
@@ -754,7 +788,7 @@ public:
 #endif
 
 protected:
-#ifndef _IMPL_NS_LAYOUT
+#ifndef MOZILLA_INTERNAL_API
   nsCOMPtr<nsPIDOMWindow> mWindow;
 #endif
   PopupControlState mOldState;
